<?php
/**
 * This file is part of the official Amazon Checkout extension for Magento 1.x
 *
 * (c) 2021 creativestyle GmbH. All Rights reserved
 *
 * Distribution of the derivatives reusing, transforming or being built upon
 * this software, is not allowed without explicit written permission granted
 * by creativestyle GmbH
 *
 * @category   Creativestyle
 * @package    Creativestyle_AmazonCheckout
 * @copyright  2021 creativestyle GmbH
 * @author     Marek Stefan <ticket@creativestyle.de>
 */


class Creativestyle_AmazonCheckout_Model_Debug
{
    const APPLIED_PATCHES_FILENAME = 'applied.patches.list';
    const REQUIREMENTS_FILENAME = 'requirements.json';

    /**
     * Store views collection
     */
    protected $_storeCollection = null;

    /**
     * Reference timestamp
     */
    protected $_currentTimestamp = null;

    /**
     * @var array
     */
    protected $_configurationToCollect = array(
        Mage_Core_Model_Url::XML_PATH_SECURE_IN_FRONT,
        Mage_Core_Model_Locale::XML_PATH_DEFAULT_LOCALE,
        Mage_Core_Model_Locale::XML_PATH_DEFAULT_TIMEZONE,
        Mage_Checkout_Helper_Data::XML_PATH_GUEST_CHECKOUT,
        Mage_Directory_Model_Currency::XML_PATH_CURRENCY_BASE,
        Mage_Core_Model_Session_Abstract::XML_PATH_USE_REMOTE_ADDR,
        Mage_Core_Model_Session_Abstract::XML_PATH_USE_HTTP_VIA,
        Mage_Core_Model_Session_Abstract::XML_PATH_USE_X_FORWARDED,
        Mage_Core_Model_Session_Abstract::XML_PATH_USE_USER_AGENT,
        Mage_Core_Model_Session_Abstract::XML_PATH_USE_FRONTEND_SID,
        'sales/minimum_order/active',
        'sales/minimum_order/amount'
    );

    /**
     * @var array
     */
    protected $_configFilters = array();

    public function __construct()
    {
        $this->_configFilters = array(
            Mage_Core_Model_Url::XML_PATH_SECURE_IN_FRONT => array($this, '_convertToBool'),
            Mage_Checkout_Helper_Data::XML_PATH_GUEST_CHECKOUT => array($this, '_convertToBool'),
            Mage_Core_Model_Session_Abstract::XML_PATH_USE_REMOTE_ADDR => array($this, '_convertToBool'),
            Mage_Core_Model_Session_Abstract::XML_PATH_USE_HTTP_VIA => array($this, '_convertToBool'),
            Mage_Core_Model_Session_Abstract::XML_PATH_USE_X_FORWARDED => array($this, '_convertToBool'),
            Mage_Core_Model_Session_Abstract::XML_PATH_USE_USER_AGENT => array($this, '_convertToBool'),
            Mage_Core_Model_Session_Abstract::XML_PATH_USE_FRONTEND_SID => array($this, '_convertToBool'),
            'sales/minimum_order/active' => array($this, '_convertToBool'),
            'amazonpay/account/use_legacy' => array($this, '_convertToBool'),
            'amazonpay/general/product_page_active' => array($this, '_convertToBool'),
            'amazonpay/general/active' => array($this, '_convertToBool'),
            'amazonpay/general/login_active' => array($this, '_convertToBool'),
            'amazonpay/delivery/restrict_packstation' => array($this, '_convertToBool'),
            'amazonpay/delivery/restrict_po_box' => array($this, '_convertToBool'),
            'amazonpay/developer/log_active' => array($this, '_convertToBool'),
            'amazonpay/advanced/show' => array($this, '_convertToBool'),
            'amazonpay/alexa/active' => array($this, '_convertToBool'),
            'amazonpay/account/private_key' => array($this, '_validatePrivateKey'),
            'amazonpay/alexa/carrier_codes' => 'unserialize'
        );
    }

    protected function _getAppliedPatchesPath()
    {
        return Mage::getBaseDir('etc') . DS . self::APPLIED_PATCHES_FILENAME;
    }

    /**
     * Returns store views collection
     *
     * @return Mage_Core_Model_Resource_Store_Collection
     */
    protected function _getStoreCollection()
    {
        if (null === $this->_storeCollection) {
            $this->_storeCollection = Mage::getModel('core/store')->getCollection()->load();
        }

        return $this->_storeCollection;
    }

    protected function _convertToBool($value)
    {
        return (bool)$value;
    }

    protected function _validatePrivateKey($value)
    {
        return 'VALID';
    }

    protected function _filterConfigValue($key, $value)
    {
        if (array_key_exists($key, $this->_configFilters)) {
            // @codingStandardsIgnoreStart
            return call_user_func_array($this->_configFilters[$key], array($value));
            // @codingStandardsIgnoreEnd
        }

        return $value;
    }

    /**
     * @param Mage_Core_Model_Store|null $store
     * @return string
     */
    protected function _getStoreViewBaseUrl($store = null)
    {
        return Mage::getModel('core/url')->setStore($store)->getUrl(
            '/',
            array('_current' => false, '_nosid' => true, '_store' => $store ? $store->getId() : null)
        );
    }

    /**
     * @param Mage_Core_Model_Store|null $store
     * @return string
     */
    protected function _getStoreViewSecureBaseUrl($store = null)
    {
        return Mage::getModel('core/url')->setStore($store)->getUrl(
            '/',
            array('_current' => false, '_secure' => true, '_nosid' => true, '_store' => $store ? $store->getId() : null)
        );
    }

    /**
     * @param Mage_Core_Model_Store|null $store
     * @return array
     */
    protected function _getStoreViewConfig(Mage_Core_Model_Store $store = null)
    {
        $storeCode = $store ? $store->getCode() : null;
        $storeId = $store ? $store->getId() : Mage_Core_Model_App::ADMIN_STORE_ID;
        $groupId = $store ? $store->getGroupId() : null;
        $websiteId = $store ? $store->getWebsiteId() : null;

        $config = array(
            'code' => $storeCode,
            'id' => $storeId,
            'group_id' => $groupId,
            'website_id' => $websiteId,
            'base_url' => $this->_getStoreViewBaseUrl($store),
            'secure_base_url' => $this->_getStoreViewSecureBaseUrl($store)
        );

        foreach ($this->_configurationToCollect as $configPath) {
            $config[$configPath] = Mage::getStoreConfig($configPath, $storeId);
        }

        foreach (Mage::getStoreConfig('amazonpay', $storeId) as $group => $configOptions) {
            foreach ($configOptions as $configKey => $configValue) {
                $config[sprintf('%s/%s/%s', 'amazonpay', $group, $configKey)] = $configValue;
            }
        }

        foreach ($config as $key => $value) {
            $config[$key] = $this->_filterConfigValue($key, $value);
        }
        return $config;
    }

    /**
     * Returns edition of Magento
     *
     * @return string
     */
    protected function _getMagentoEdition()
    {
        if (method_exists('Mage', 'getEdition')) {
            return Mage::getEdition();
        }

        return '';
    }

    protected function _getMagentoVersion()
    {
        return Mage::getVersion();
    }

    /**
     * @return string|null
     */
    protected function _getOpenMageVersion()
    {
        if (method_exists('Mage', 'getOpenMageVersion')) {
            return Mage::getOpenMageVersion();
        }

        return null;
    }

    /**
     * Gets and returns current timestamp
     *
     * @return int
     */
    protected function _getCurrentTimestamp()
    {
        if (null === $this->_currentTimestamp) {
            $this->_currentTimestamp = Mage::getSingleton('core/date')->gmtTimestamp();
        }

        return $this->_currentTimestamp;
    }

    /**
     * Formats input date into date with timezone offset
     * Input date must be in GMT timezone
     *
     * @param  string $format
     * @param  int|string $input date in GMT timezone
     * @return string
     */
    protected function _formatDate($format, $input)
    {
        // @codingStandardsIgnoreStart
        return Mage::getSingleton('core/date')->date($format, $input);
        // @codingStandardsIgnoreEnd
    }

    /**
     * Return dummy RSA private key for debugging purposes
     *
     * @return string
     */
    protected function _getDummyPrivateKey()
    {
        return <<<EOF
-----BEGIN PRIVATE KEY-----
MIIEvgIBADANBgkqhkiG9w0BAQEFAASCBKgwggSkAgEAAoIBAQCmPJ4mOBP8fG3y
Ixa6b5kDJ5WhrU+UYGIc+s2RdfDJ8kovE+EcYM3gAcbnb7IHMVpL6q+GmzcpfFH4
1X/XHM68rSpVZM43MLpbOo+I+0kI3CGmPtLe01+/qwuTviKjQ0A2hGEcH9oMkLVx
TMDyheXVbvPeqStZIFT2TELdk3niD0E/g9nJ04UiRMLepKI3FKXBqNbO7mU7VAFs
ho4cq5dVyt4gXKQOnc6GXnAF5hPKE2q1MnKUp80VahA+r7+LYeT7tm7hLSOsrnv6
w/OASfF9QJEwN731nkSZj1bzpk5SmhHCKwnEMxFuRWUWDU3DvjSOcTxJwvm7hpT7
xaDTqxZJAgMBAAECggEAMZaEgJca0ybU7zVrWrredHPy7TMTOYoUyltARgNeDAej
/G4cKQhxcUBmcnQ9sWqV6m575JXZex0UILujMmpr+xlWtvlmkE0PUhZiabS3wRZV
TU+RPJnmBnlB71T45SWy+nA7vZV8yGHEViNUrSOm/ydlRMbG4QPFhGEUUx9ibL2v
UwJcnz/dDNNV+fyjoDEuo27QEvOagAoZSmZ/JrhLAN2FefrLwWMbZpEioZrMKReU
gy504YJZ98mlwWLHWYdRhyGmICrIJKLbgm8v3SnyYLN7UJLcuzhih54Z61c18zQI
n7vUsdhkH/vF+IDoQirx1V1523oQEUwpMx/rrtKnzwKBgQDnOiadrSTssdHBQwip
MMpg/ItGNBnBxeMim2HIj2VmaJnK+EN18JSgC2CjkOAaRUFb5+yM5KhydfdmNBgZ
WOy+xfDNs/5LI+E5yWfsKK7mD0fdNEIBg8KCv2JLuStVS827KTZGE2IctyvwSfCf
xWABux4Rx14SRDLvIEF5YrTwBwKBgQC4C/s0RfNzj7VawQ8xoTgaAcJHn4Qrq7LW
pvsLPYvVcWrzOXwivfC1uRL71OEg50PseFA1bFUva7VMIWI7VOUpr/p6PbQYm+77
lLi0UnSLfxNKzFZ28TE85if4FZjA4g3PDfd9NhbPt5DN6VXFMLwuWyX48GE6/oJ2
YMGDVqmTLwKBgBhFO05roSeTFqBBiAQyUdvSRlAtIdL8EXZNdvsVSiBwtEvguTwE
76TI7PDeQReg01ZxRpwXWDMDfN9mcm0er2Lq5fbBgNykPHzTTiw0rP8K9ZEbbh44
lU8iBZBLNGg0l7igf8jlOfQnqd4D9NTgY85qtKJLGe2y6OG+sgCqwrE7AoGBAIPT
oST6DbzybHYK3rUvwx1tsRjYOzWEkF6L0Dlm5igGSxLYVAAWiLnBTMWw32hP/ij8
rIb3h7Sls4BKUTAqSNKpSAIS/FK8LltpVTtj7DKW3NywrCYZ8KisOmjKH1XZFQb/
/AtlmMVDuKakwViy5bdWdkZu91XoQs5D05gdlOzlAoGBALQOJfIxBIFsq6bAfcBl
nCTVzZIHtOgoTSLSK+87reQC37OJceCHfkI4wIrtLMYCOnDCdrZPbX2MEuwB3n44
aXgfFgcpKHLQoC7PRsuEEBQy51tQPA1cSywpDg30Lh9oVBwV5LTKx1chz3XUV20z
EZuWFh/rMAhLPH5sK/g3zLbJ
-----END PRIVATE KEY-----
EOF;
    }

    /**
     * Validate installed phpseclib
     *
     * @return bool
     */
    public function validatePhpseclib()
    {
        try {
            if (class_exists('Crypt_RSA', false)) {
                $rsa = new Crypt_RSA();
            } elseif (class_exists('\phpseclib\Crypt\RSA', true)) {
                $rsa = new \phpseclib\Crypt\RSA();
            } else {
                return false;
            }

            $rsa->setHash(AmazonPayV2_Client::HASH_ALGORITHM);
            $rsa->setMGFHash(AmazonPayV2_Client::HASH_ALGORITHM);
            $rsa->setSaltLength(20);

            $rsa->loadKey($this->_getDummyPrivateKey());

            if (empty($rsa->modulus) || empty($rsa->exponent)) {
                return false;
            }

        } catch (Exception $e) {
            return false;
        }

        return true;
    }

    public function getGeneralData()
    {
        return array(
            'extension_version' => (string)Mage::getConfig()->getNode('modules/Creativestyle_AmazonCheckout/version'),
            'sdk_version' => AmazonPayV2_Client::SDK_VERSION,
            'magento_version' => $this->_getMagentoVersion(),
            'magento_edition' => $this->_getMagentoEdition(),
            'openmage_version' => $this->_getOpenMageVersion(),
            'php_version' => PHP_VERSION,
            'missing_patches' => join(', ', $this->getMissingPatches()),
            'suhosin_installed' => defined('SUHOSIN_PATCH') || extension_loaded('suhosin'),
            'phpseclib_valid' => $this->validatePhpseclib(),
            'magento_compiler_enabled' => defined('COMPILER_INCLUDE_PATH'),
            'amazon_user_id_present' => Mage::getResourceModel('catalog/eav_attribute')
                ->loadByCode('customer', 'amazon_user_id')->getId() ? true : false,
            'timestamp' => $this->_getCurrentTimestamp(),
            'timestamp_human' => $this->_formatDate("Y-m-d H:i:s", $this->_getCurrentTimestamp())
        );

    }

    public function getStoreViews()
    {
        $config = array(
            Mage_Core_Model_App::ADMIN_STORE_ID => $this->_getStoreViewConfig()
        );

        foreach ($this->_getStoreCollection() as $store) {
            $config[$store->getCode()] = $this->_getStoreViewConfig($store);
        }

        return $config;
    }

    /**
     * @return array
     */
    public function getRequiredPatches()
    {
        try {
            $jsonPath = Mage::getModuleDir('data', 'Creativestyle_AmazonCheckout') . DS . self::REQUIREMENTS_FILENAME;
            $jsonData = file_get_contents($jsonPath);
            $requirements = json_decode($jsonData, true);
            $magentoVer = $this->_getMagentoVersion();
            if (isset($requirements[$magentoVer]) && !empty($requirements[$magentoVer])) {
                return $requirements[$magentoVer];
            }
        } catch (Exception $e) {
            return array();
        }

        return array();
    }


    public function getAppliedPatches()
    {
        $appliedPatches = array();
        $appliedPatchesPath = $this->_getAppliedPatchesPath();
        $ioAdapter = new Varien_Io_File();

        if ($ioAdapter->fileExists($appliedPatchesPath)) {
            $ioAdapter->open(array('path' => $ioAdapter->dirname($appliedPatchesPath)));
            $ioAdapter->streamOpen($appliedPatchesPath, 'r');

            while ($buffer = $ioAdapter->streamRead()) {
                if (stristr($buffer, '|')) {
                    list($date, $patch, $magentoVersion, $patchVersion) = array_map('trim', explode('|', $buffer));
                    $appliedPatches[] = array(
                        'date' => $date,
                        'patch' => $patch,
                        'version' => $patchVersion,
                        'magento' => str_replace('_', ' ', $magentoVersion)
                    );
                }
            }

            $ioAdapter->streamClose();
        }

        return $appliedPatches;
    }

    public function getMissingPatches()
    {
        $requiredPatches = $this->getRequiredPatches();
        $appliedPatches = array_map(
            function ($patch) {
                $patchExploded = explode('_', $patch['patch']);
                $version = (strtolower($patch['version']) != 'v1') ? strtolower($patch['version']) : '';
                return $patchExploded[0] . $version;
            },
            $this->getAppliedPatches()
        );
        return array_diff($requiredPatches, $appliedPatches);
    }

    /**
     * Returns array of installed Magento extensions
     *
     * @return array
     */
    public function getInstalledExtensions()
    {
        $extensions = array();
        $modules = Mage::getConfig()->getNode('modules')->asArray();
        foreach ($modules as $key => $data) {
            $extensions[$key] = isset($data['active']) && ($data['active'] == 'false' || !$data['active'])
                ? false
                : (isset($data['version']) && $data['version'] ? $data['version'] : true);
        }

        return $extensions;
    }

    public function getCompleteDebugData()
    {
        return array_merge(
            $this->getGeneralData(),
            array(
                'config' => $this->getStoreViews(),
                'patches' => $this->getAppliedPatches(),
                'extensions' => $this->getInstalledExtensions(),
            )
        );
    }

    /**
     * Return descriptive Magento version string for reporting purposes
     *
     * @return string
     */
    public function getMagentoVersionString()
    {
        if ($this->_getOpenMageVersion()) {
            return 'OpenMage ' . $this->_getOpenMageVersion();
        }

        return 'Magento ' . $this->_getMagentoVersion();
    }
}
